@@ -32,7 +32,6 @@
 #include "khtml_part.h"
 #include "html/html_documentimpl.h"
 #include "khtml_settings.h"
-#include "misc/htmlhashes.h"
 
 #include "css/cssstyleselector.h"
 #include "css/cssproperties.h"
@@ -138,7 +137,7 @@ static QByteArray encodeCString(const QByteArray& e)
 {
     // http://www.w3.org/TR/html4/interact/forms.html#h-17.13.4.1
     // safe characters like NS handles them for compatibility
-    static const char *safe = "-._*";
+    static const char safe[] = "-._*";
     QByteArray encoded(( e.length()+e.count( '\n' ) )*3
                      +e.count('\r') * 3 + 1, 0);
     int enclen = 0;
@@ -270,6 +269,8 @@ QByteArray HTMLFormElementImpl::formData(bool& ok)
 
     QByteArray enc_string = ""; // used for non-multipart data
 
+    bool useMultipart = m_multipart && m_post; // as multipart is ignored for GET
+
     // find out the QTextcodec to use
     QString str = m_acceptcharset.string();
     const QChar space(' ');
@@ -333,14 +334,14 @@ QByteArray HTMLFormElementImpl::formData(bool& ok)
         HTMLGenericFormElementImpl* const current = ordered[i];
         khtml::encodingList lst;
 
-        if (!current->disabled() && current->encoding(codec, lst, m_multipart))
+        if (!current->disabled() && current->encoding(codec, lst, useMultipart))
         {
             //kDebug(6030) << "adding name '" << current->name().string() << "'";
             khtml::encodingList::ConstIterator it = lst.constBegin();
             const khtml::encodingList::ConstIterator itEnd = lst.constEnd();
             for( ; it != itEnd; ++it )
             {
-                if (!m_multipart)
+                if (!useMultipart)
                 {
                     // handle ISINDEX / <input name=isindex> special
                     // but only if its the first entry
@@ -446,7 +447,7 @@ QByteArray HTMLFormElementImpl::formData(bool& ok)
         }
     }
 
-    if (m_multipart)
+    if (useMultipart)
         enc_string = QString("--" + m_boundary + "--\r\n").toAscii().constData();
 
     const int old_size = form_data.size();
@@ -463,7 +464,6 @@ void HTMLFormElementImpl::setEnctype( const DOMString& type )
     {
         m_enctype = "multipart/form-data";
         m_multipart = true;
-        m_post = true;
     } else if (type.string().indexOf("text", 0, Qt::CaseInsensitive) != -1 || type.string().indexOf("plain", 0, Qt::CaseInsensitive) != -1)
     {
         m_enctype = "text/plain";
@@ -824,6 +824,15 @@ void HTMLFormElementImpl::removeFormElement(HTMLGenericFormElementImpl *e)
     int i = formElements.indexOf(e);
     if (i != -1)
         formElements.removeAt(i);
+        
+    if (e->hasPastNames()) {
+        QMutableHashIterator<DOMString, HTMLGenericFormElementImpl*> it(m_pastNamesMap);
+        while (it.hasNext()) {
+            it.next();
+            if (it.value() == e)
+                it.remove();
+        }
+    }
 }
 
 void HTMLFormElementImpl::registerImgElement(HTMLImageElementImpl *e)
@@ -838,12 +847,29 @@ void HTMLFormElementImpl::removeImgElement(HTMLImageElementImpl *e)
         imgElements.removeAt(i);
 }
 
+HTMLGenericFormElementImpl* HTMLFormElementImpl::lookupByPastName(const DOMString& id)
+{
+    return m_pastNamesMap.value(id);
+}
+
+void HTMLFormElementImpl::bindPastName(HTMLGenericFormElementImpl* e)
+{
+    DOMString id = e->getAttribute(ATTR_ID);
+    if (!id.isEmpty())
+        m_pastNamesMap.insert(id, e);
+        
+    DOMString nm = e->getAttribute(ATTR_NAME);
+    if (!nm.isEmpty())
+        m_pastNamesMap.insert(nm, e);
+    e->setHasPastNames();
+}
+
 // -------------------------------------------------------------------------
 
 HTMLGenericFormElementImpl::HTMLGenericFormElementImpl(DocumentImpl *doc, HTMLFormElementImpl *f)
     : HTMLElementImpl(doc)
 {
-    m_disabled = m_readOnly = false;
+    m_disabled = m_readOnly = m_hasPastNames = false;
     m_name = 0;
 
     if (f)
@@ -1085,7 +1111,8 @@ void HTMLGenericFormElementImpl::defaultEventHandler(EventImpl *evt)
 	    // handle tabbing out, either from a single or repeated key event.
 	    if ( evt->id() == EventImpl::KEYPRESS_EVENT && evt->isKeyRelatedEvent() ) {
 	        QKeyEvent* const k = static_cast<KeyEventBaseImpl *>(evt)->qKeyEvent();
-	        if ( k && (k->key() == Qt::Key_Tab || k->key() == Qt::Key_Backtab) ) {
+	        if ( k && (k->key() == Qt::Key_Tab || k->key() == Qt::Key_Backtab) &&
+                        (k->modifiers() & Qt::ControlModifier) == 0 ) {
 		    QWidget* const widget = static_cast<RenderWidget*>(m_render)->widget();
 		    if (widget)
 			{
@@ -1514,6 +1541,9 @@ void HTMLInputElementImpl::copyNonAttributeProperties(const ElementImpl* source)
 
     m_value = e->m_value;
     m_checked = e->m_checked;
+    m_defaultChecked = e->m_checked;
+    m_useDefaultChecked = e->m_defaultChecked;
+    m_indeterminate = e->m_indeterminate;
     // ### copy more?
 
     HTMLGenericFormElementImpl::copyNonAttributeProperties(source);
@@ -2205,9 +2235,9 @@ void HTMLSelectElementImpl::add( HTMLElementImpl* element, HTMLElementImpl* befo
     if (before == 0 && (m_multiple || !option->selectedBit()) && !m_recalcListItems)
         fastAppendLast = true;
 
-    insertBefore(option, before, exceptioncode );
+    insertBefore(option, before, exceptioncode);
 
-    if (fastAppendLast) {
+    if (fastAppendLast && !exceptioncode) {
         m_listItems.resize(m_listItems.size() + 1);
         m_listItems[m_listItems.size() - 1] = option;
         ++m_length;
@@ -2247,7 +2277,7 @@ void HTMLSelectElementImpl::remove( long index )
         setRecalcListItems();
 }
 
-DOMString HTMLSelectElementImpl::value( ) const
+HTMLOptionElementImpl* HTMLSelectElementImpl::firstSelectedItem() const
 {
     uint i;
     const QVector<HTMLGenericFormElementImpl*> items = listItems();
@@ -2255,8 +2285,16 @@ DOMString HTMLSelectElementImpl::value( ) const
     for (i = 0; i < itemsSize; ++i) {
         if ( items[i]->id() == ID_OPTION
             && static_cast<HTMLOptionElementImpl*>(items[i])->selectedBit())
-            return static_cast<HTMLOptionElementImpl*>(items[i])->value();
+            return static_cast<HTMLOptionElementImpl*>(items[i]);
     }
+    return 0;
+}
+
+DOMString HTMLSelectElementImpl::value( ) const
+{
+    HTMLOptionElementImpl* o = firstSelectedItem();
+    if (o)
+        return o->value();
     return DOMString("");
 }
 
@@ -2279,6 +2317,15 @@ QString HTMLSelectElementImpl::state( )
 
     const int l = items.count();
 
+    // We have two encoding schemes: for single-select ones, if the selected
+    // element has an ID, we encoded it as iFoo.
+    if (!multiple()) {
+        HTMLOptionElementImpl* item = firstSelectedItem();
+        if (item && item->hasID())
+            return QString::fromLatin1("i") + item->getAttribute(ATTR_ID).string();
+    }
+
+    // Otherwise we merely go positional..
     state.fill('.', l);
     for(int i = 0; i < l; ++i)
         if(items[i]->id() == ID_OPTION && static_cast<HTMLOptionElementImpl*>(items[i])->selectedBit())
@@ -2292,14 +2339,39 @@ void HTMLSelectElementImpl::restoreState(const QString &_state)
     recalcListItems();
 
     QString state = _state;
+    const QVector<HTMLGenericFormElementImpl*> items = listItems();
+    const int l = items.count();
+
+    // First see if we have an id-tagged one, and if it's correct.
+    if (state.startsWith(QLatin1Char('i'))) {
+        DOM::DOMString id = state.mid(1);
+        DOM::ElementImpl* cand = document()->getElementById(id);
+
+        // No such ID or not an option -> wrong
+        if (!cand || cand->id() != ID_OPTION)
+            return;
+
+        // See if it's one of ours, and select if so.
+        for (int i = 0; i < l; ++i) {
+            if (items[i] == cand)
+                static_cast<HTMLOptionElementImpl*>(cand)->setSelected(true);
+        }
+
+        // If we succeeded, our state is updated --- if not, we better
+        // leave this be.
+        return;
+    }
+    
     if(!state.isEmpty() && !state.contains('X') && !m_multiple && m_size <= 1) {
         qWarning("should not happen in restoreState!");
         state[0] = 'X';
     }
 
-    const QVector<HTMLGenericFormElementImpl*> items = listItems();
+    // Now we have positional encoding. Make sure the length matches.
+    if (m_length != state.length()) // m_length == number of <option>, while
+                                    // l == items.length() includes optgroups, too.
+        return;
 
-    const int l = items.count();
     for(int i = 0; i < l; ++i) {
         if(items[i]->id() == ID_OPTION) {
             HTMLOptionElementImpl* const oe = static_cast<HTMLOptionElementImpl*>(items[i]);
