@@ -248,6 +248,7 @@ SlaveBase::~SlaveBase()
 {
     delete d->configGroup;
     delete d->config;
+    delete d->remotefile;
     delete d;
     s_protocol = "";
 }
@@ -375,11 +376,11 @@ void SlaveBase::sendAndKeepMetaData()
 
 KRemoteEncoding *SlaveBase::remoteEncoding()
 {
-   if (d->remotefile != 0)
+   if (d->remotefile)
       return d->remotefile;
 
-   QByteArray charset = metaData("Charset").toLatin1();
-   return d->remotefile = new KRemoteEncoding( charset );
+   const QByteArray charset (metaData("Charset").toLatin1());
+   return (d->remotefile = new KRemoteEncoding( charset ));
 }
 
 void SlaveBase::data( const QByteArray &data )
@@ -587,7 +588,7 @@ void SlaveBase::mimeType( const QString &_type)
     // Send the meta-data each time we send the mime-type.
     if (!mOutgoingMetaData.isEmpty())
     {
-      // kDebug(7019) << "mimeType: emitting meta data";
+      // kDebug(7019) << "emitting meta data";
       KIO_DATA << mOutgoingMetaData;
       send( INF_META_DATA, data );
     }
@@ -816,6 +817,8 @@ void SlaveBase::slave_status()
 
 void SlaveBase::reparseConfiguration()
 {
+    delete d->remotefile;
+    d->remotefile = 0;
 }
 
 bool SlaveBase::openPasswordDialog( AuthInfo& info, const QString &errorMsg )
@@ -979,6 +982,7 @@ void SlaveBase::dispatch( int command, const QByteArray &data )
         disconnectSlave();
         d->isConnectedToApp = true;
         connectSlave(app_socket);
+        virtual_hook(AppConnectionMade, 0);
     } break;
     case CMD_SLAVE_HOLD: {
         KUrl url;
